#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableClass]
  public sealed class RegressionProblemData : DataAnalysisProblemData,
    IRegressionProblemData {
    private const string TargetVariableParameterName = "Target variable";

    #region parameter properties
    public IValueParameter<StringValue> TargetVariableParameter {
      get { return (IValueParameter<StringValue>)Parameters[TargetVariableParameterName]; }
    }
    #endregion
    #region propeties
    public string TargetVariable {
      get { return TargetVariableParameter.Value.Value; }
    }
    #endregion

    private RegressionProblemData(RegressionProblemData original, Cloner cloner)
      : base(original, cloner) {
      RegisterEventHandlers();
    }
    [StorableConstructor]
    private RegressionProblemData(bool deserializing) : base(deserializing) { }
    public RegressionProblemData()
      : base() {
      ConstrainedValueParameter<StringValue> targetVariableParameter = new ConstrainedValueParameter<StringValue>(TargetVariableParameterName);
      foreach (var variableName in Dataset.VariableNames)
        targetVariableParameter.ValidValues.Add(new StringValue(variableName));
      targetVariableParameter.Value = targetVariableParameter.ValidValues.First();

      Parameters.Add(targetVariableParameter);

      RegisterEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RegressionProblemData(this, cloner);
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEventHandlers();
    }

    #region event propagation
    private void RegisterEventHandlers() {
      TargetVariableParameter.Value.ValueChanged += new EventHandler(TargetVariable_ValueChanged);
      TargetVariableParameter.ValueChanged += new EventHandler(TargetVariableParameter_ValueChanged);
    }

    private void TargetVariableParameter_ValueChanged(object sender, EventArgs e) {
      TargetVariableParameter.Value.ValueChanged += new EventHandler(TargetVariable_ValueChanged);
      OnProblemChanged();
    }

    private void TargetVariable_ValueChanged(object sender, EventArgs e) {
      OnProblemChanged();
    }
    #endregion
  }
}
